@@ -20,67 +20,6 @@ import select
 from MAVProxy.modules.lib import textconsole
 from MAVProxy.modules.lib import mp_settings
 
-class MPSettings(object):
-    def __init__(self):
-        self.vars = [ ('link', int),
-                      ('altreadout', int),
-                      ('distreadout', int),
-                      ('battreadout', int),
-                      ('heartbeat', int),
-                      ('numcells', int),
-                      ('speech', int),
-                      ('mavfwd', int),
-                      ('mavfwd_rate', int),
-                      ('streamrate', int),
-                      ('streamrate2', int),
-                      ('heartbeatreport', int),
-                      ('radiosetup', int),
-                      ('paramretry', int),
-                      ('moddebug', int),
-                      ('rc1mul', int),
-                      ('rc2mul', int),
-                      ('rc4mul', int)]
-        self.link = 1
-        self.altreadout = 10
-        self.distreadout = 200
-        self.battreadout = 0
-        self.basealtitude = -1
-        self.heartbeat = 1
-        self.numcells = 0
-        self.mavfwd = 1
-        self.mavfwd_rate = 0
-        self.speech = 0
-        self.streamrate = 4
-        self.streamrate2 = 4
-        self.radiosetup = 0
-        self.heartbeatreport = 1
-        self.paramretry = 10
-        self.rc1mul = 1
-        self.rc2mul = 1
-        self.rc4mul = 1
-        self.moddebug = 0
-
-    def set(self, vname, value):
-        '''set a setting'''
-        for (v,t) in sorted(self.vars):
-            if v == vname:
-                try:
-                    value = t(value)
-                except:
-                    print("Unable to convert %s to type %s" % (value, t))
-                    return
-                setattr(self, vname, value)
-                return
-
-    def show(self, v):
-        '''show settings'''
-        print("%20s %s" % (v, getattr(self, v)))
-
-    def show_all(self):
-        '''show all settings'''
-        for (v,t) in sorted(self.vars):
-            self.show(v)
-
 class MPStatus(object):
     '''hold status information about the mavproxy'''
     def __init__(self):
@@ -105,6 +44,7 @@ class MPStatus(object):
         self.mav_error = 0
         self.target_system = 1
         self.target_component = 1
+        self.rallyloader = mavwp.MAVRallyLoader(self.target_system, self.target_component)
         self.speech = None
         self.altitude = 0
         self.last_altitude_announce = 0.0
@@ -119,6 +59,7 @@ class MPStatus(object):
         self.last_override = [ 0 ] * 8
         self.override_counter = 0
         self.flightmode = 'MAV'
+        self.last_mode_announce = 0
         self.logdir = None
         self.last_heartbeat = 0
         self.last_message = 0
@@ -167,6 +108,7 @@ class MPState(object):
     def __init__(self):
         self.console = textconsole.SimpleConsole()
         self.map = None
+        self.map_functions = {}
         self.settings = mp_settings.MPSettings(
             [ ('link', int, 1),
               ('altreadout', int, 10),
@@ -185,7 +127,9 @@ class MPState(object):
               ('rc2mul', int, 1),
               ('rc4mul', int, 1),
               ('shownoise', int, 1),
-              ('basealt', int, 0)]
+              ('basealt', int, 0),
+              ('wpalt', int, 100),
+              ('parambatch', int, 10)]
             )
         self.status = MPStatus()
 
@@ -504,6 +448,46 @@ def save_waypoints(filename):
         return
     print("Saved %u waypoints to %s" % (mpstate.status.wploader.count(), filename))
 
+def wp_draw_callback(points):
+    '''callback from drawing waypoints'''
+    if len(points) < 3:
+        return
+    from MAVProxy.modules.lib import mp_util
+    home = mpstate.status.wploader.wp(0)
+    mpstate.status.wploader.clear()
+    mpstate.status.wploader.target_system = mpstate.status.target_system
+    mpstate.status.wploader.target_component = mpstate.status.target_component
+    mpstate.status.wploader.add(home)
+    for p in points:
+        mpstate.status.wploader.add_latlonalt(p[0], p[1], mpstate.settings.wpalt)
+    mpstate.master().waypoint_clear_all_send()
+    if mpstate.status.wploader.count() == 0:
+        return
+    mpstate.status.loading_waypoints = True
+    mpstate.status.loading_waypoint_lasttime = time.time()
+    mpstate.master().waypoint_count_send(mpstate.status.wploader.count())
+
+def set_home_location():
+    '''set home location from last map click'''
+    try:
+        latlon = mpstate.map_state.click_position
+    except Exception:
+        print("No map available")
+        return
+    lat = float(latlon[0])
+    lon = float(latlon[1])
+    if mpstate.status.wploader.count() == 0:
+        mpstate.status.wploader.add_latlonalt(lat, lon, 0)
+    w = mpstate.status.wploader.wp(0)
+    w.x = lat
+    w.y = lon
+    mpstate.status.wploader.set(w, 0)
+    mpstate.status.loading_waypoints = True
+    mpstate.status.loading_waypoint_lasttime = time.time()
+    mpstate.master().mav.mission_write_partial_list_send(mpstate.status.target_system,
+                                                         mpstate.status.target_component,
+                                                         0, 0)
+    
 
 def cmd_wp(args):
     '''waypoint commands'''
@@ -547,6 +531,17 @@ def cmd_wp(args):
         mpstate.master().waypoint_set_current_send(int(args[1]))
     elif args[0] == "clear":
         mpstate.master().waypoint_clear_all_send()
+    elif args[0] == "draw":
+        if not 'draw_lines' in mpstate.map_functions:
+            print("No map drawing available")
+            return        
+        if mpstate.status.wploader.count() == 0:
+            print("Need home location - refresh waypoints")
+            return
+        mpstate.map_functions['draw_lines'](wp_draw_callback)
+        print("Drawing waypoints on map")
+    elif args[0] == "sethome":
+        set_home_location()        
     else:
         print("Usage: wp <list|load|save|set|show|clear>")
 
@@ -570,17 +565,8 @@ def fetch_fence_point(i):
 
 
 
-def load_fence(filename):
-    '''load fence points from a file'''
-    try:
-        mpstate.status.fenceloader.target_system = mpstate.status.target_system
-        mpstate.status.fenceloader.target_component = mpstate.status.target_component
-        mpstate.status.fenceloader.load(filename)
-    except Exception, msg:
-        print("Unable to load %s - %s" % (filename, msg))
-        return
-    print("Loaded %u geo-fence points from %s" % (mpstate.status.fenceloader.count(), filename))
-
+def send_fence():
+    '''send fence points from fenceloader'''
     # must disable geo-fencing when loading
     action = get_mav_param('FENCE_ACTION', mavutil.mavlink.FENCE_ACTION_NONE)
     param_set('FENCE_ACTION', mavutil.mavlink.FENCE_ACTION_NONE)
@@ -600,6 +586,18 @@ def load_fence(filename):
             return
     param_set('FENCE_ACTION', action)
 
+def load_fence(filename):
+    '''load fence points from a file'''
+    try:
+        mpstate.status.fenceloader.target_system = mpstate.status.target_system
+        mpstate.status.fenceloader.target_component = mpstate.status.target_component
+        mpstate.status.fenceloader.load(filename)
+    except Exception, msg:
+        print("Unable to load %s - %s" % (filename, msg))
+        return
+    print("Loaded %u geo-fence points from %s" % (mpstate.status.fenceloader.count(), filename))
+    send_fence()
+
 
 def list_fence(filename):
     '''list fence points, optionally saving to a file'''
@@ -632,10 +630,28 @@ def list_fence(filename):
         print("Saved fence to %s" % fencetxt)
 
 
+def fence_draw_callback(points):
+    '''callback from drawing a fence'''
+    if len(points) < 3:
+        return
+    from MAVProxy.modules.lib import mp_util
+    mpstate.status.fenceloader.clear()
+    mpstate.status.fenceloader.target_system = mpstate.status.target_system
+    mpstate.status.fenceloader.target_component = mpstate.status.target_component
+    bounds = mp_util.polygon_bounds(points)
+    (lat, lon, width, height) = bounds
+    center = (lat+width/2, lon+height/2)
+    mpstate.status.fenceloader.add_latlon(center[0], center[1])
+    for p in points:
+        mpstate.status.fenceloader.add_latlon(p[0], p[1])
+    # close it
+    mpstate.status.fenceloader.add_latlon(points[0][0], points[0][1])
+    send_fence()
+
 def cmd_fence(args):
     '''geo-fence commands'''
     if len(args) < 1:
-        print("usage: fence <list|load|save|clear>")
+        print("usage: fence <list|load|save|clear|draw>")
         return
 
     if args[0] == "load":
@@ -655,12 +671,147 @@ def cmd_fence(args):
             print("usage: fence show <filename>")
             return
         mpstate.status.fenceloader.load(args[1])
+    elif args[0] == "draw":
+        if not 'draw_lines' in mpstate.map_functions:
+            print("No map drawing available")
+            return        
+        mpstate.map_functions['draw_lines'](fence_draw_callback)
+        print("Drawing fence on map")
     elif args[0] == "clear":
         param_set('FENCE_TOTAL', 0)
     else:
-        print("Usage: fence <list|load|save|show|clear>")
+        print("Usage: fence <list|load|save|show|clear|draw>")
 
 
+def fetch_rally_point(i):
+    '''fetch one rally point'''
+    mpstate.master().mav.rally_fetch_point_send(mpstate.status.target_system,
+                                                mpstate.status.target_component, i)
+    tstart = time.time()
+    p = None
+    while time.time() - tstart < 1:
+        p = mpstate.master().recv_match(type='RALLY_POINT', blocking=False)
+        if p is not None:
+            break
+        time.sleep(0.1)
+        continue
+    if p is None:
+        mpstate.console.error("Failed to fetch rally point %u" % i)
+        return None
+    return p
+
+def send_rally_points():
+    '''send rally points from fenceloader'''
+    param_set('RALLY_TOTAL', mpstate.status.rallyloader.rally_count())
+    for i in range(mpstate.status.rallyloader.rally_count()):
+        p = mpstate.status.rallyloader.rally_point(i)
+        mpstate.master().mav.send(p)
+        
+        #Don't know why this check was being done, but not discounting yet...
+        #Will remove before committing if I don't discover why.
+        #p2 = fetch_rally_point(i)
+        #
+        #if (p.idx != p2.idx or
+        #    abs(p.lat - p2.lat) >= 0.00003 or
+        #    abs(p.lng - p2.lng) >= 0.00003):
+        #    print("Failed to send fence point %u" % i)
+        #    param_set('FENCE_ACTION', action)
+        #    return
+
+    #TODO rally land points
+
+def list_rally_points():
+    mpstate.status.rallyloader.clear()
+    rally_count = get_mav_param('RALLY_TOTAL', 0)
+    if rally_count == 0:
+        print("No rally points")
+        return
+    for i in range(int(rally_count)):
+        p = fetch_rally_point(i)
+        if p is None:
+            return
+        mpstate.status.rallyloader.append_rally_point(p)
+
+    #TODO: rally_land points
+
+    for i in range(mpstate.status.rallyloader.rally_count()):
+        p = mpstate.status.rallyloader.rally_point(i)
+        mpstate.console.writeln("lat=%f lng=%f alt=%f break_alt=%f land_dir=%f" % (p.lat * 1e-7, p.lng * 1e-7, p.alt / 100.0, p.break_alt / 100.0, p.land_dir / 100.0))
+
+    #TODO: rally_land points
+
+def cmd_rally(args):
+    '''rally point commands'''
+    #TODO: add_land arg
+    if(len(args) < 1):
+        print("Usage: rally <list|load|save|add|clear>")
+        return
+
+    elif(args[0] == "add"):
+        if (len(args) < 2):
+            print("Usage: rally add ALT <BREAK_ALT> <LAND_HDG>")
+            return
+
+        if (mpstate.status.rallyloader.rally_count() > 4):
+            print ("Only 5 rally points possible per flight plan.")
+            return
+
+        try:
+            latlon = mpstate.map_state.click_position
+        except Exception:
+            print("No map available")
+            return
+        if latlon is None:
+            print("No map click position available")
+            return
+
+        alt = float(args[1]) * 100.0;
+        break_alt = 0.0
+        land_hdg = 0.0;
+        if (len(args) > 2):
+            break_alt = float(args[2]) * 100.0;
+        if (len(args) > 3):
+            land_hdg = float(args[3]) * 100.0;
+
+        mpstate.status.rallyloader.create_and_append_rally_point(latlon[0] * 1e7, latlon[1] * 1e7, alt, break_alt, land_hdg, 0)
+
+        send_rally_points();
+
+        print("Added Rally point at %s %f" % (str(latlon), alt / 100.0))
+    
+    elif args[0] == "clear":
+        mpstate.status.rallyloader.clear()
+        param_set('RALLY_TOTAL', 0)
+
+    elif(args[0] == "list"):
+        list_rally_points()
+
+    elif(args[0] == "load"):
+        if (len(args) < 2):
+            print("Usage: rally load filename")
+
+        try:
+            mpstate.status.rallyloader.load(args[1])
+        except Exception, msg:
+            print("Unable to load %s - %s" % (args[1], msg))
+            return
+    
+        send_rally_points()
+
+        print("Loaded %u rally points from %s" % (mpstate.status.rallyloader.rally_count(), args[1]))
+
+    elif(args[0] == "save"):
+        if (len(args) < 2):
+            print("Usage: rally save filename");
+
+        mpstate.status.rallyloader.save(args[1]);
+
+        print "Saved rally file ", args[1];
+
+    else:
+        #TODO: add_land arg
+        print("Usage: rally <list|load|save|add|clear>")
+
 def param_set(name, value, retries=3):
     '''set a parameter'''
     name = name.upper()
@@ -927,6 +1078,7 @@ command_map = {
     'rc'      : (cmd_rc,       'override a RC channel value'),
     'wp'      : (cmd_wp,       'waypoint management'),
     'fence'   : (cmd_fence,    'geo-fence management'),
+    'rally'   : (cmd_rally,    'rally point management'),
     'param'   : (cmd_param,    'manage APM parameters'),
     'setup'   : (cmd_setup,    'go into setup mode'),
     'reset'   : (cmd_reset,    'reopen the connection to the MAVLink master'),
@@ -1332,8 +1484,9 @@ def master_callback(m, master):
 
     elif mtype == "SYS_STATUS":
         battery_update(m)
-        if master.flightmode != mpstate.status.flightmode:
+        if master.flightmode != mpstate.status.flightmode and time.time() > mpstate.status.last_mode_announce + 2:
             mpstate.status.flightmode = master.flightmode
+            mpstate.status.last_mode_announce = time.time()
             mpstate.rl.set_prompt(mpstate.status.flightmode + "> ")
             say("Mode " + mpstate.status.flightmode)
 
@@ -1606,9 +1759,11 @@ def periodic_tasks():
         elif mpstate.mav_param_count != 0 and len(mpstate.mav_param_set) != mpstate.mav_param_count:
             if mpstate.master().time_since('PARAM_VALUE') >= 1:
                 diff = set(range(mpstate.mav_param_count)).difference(mpstate.mav_param_set)
-                if len(diff) > 0:
+                count = 0
+                while len(diff) > 0 and count < mpstate.settings.parambatch:
                     idx = diff.pop()
                     mpstate.master().param_fetch_one(idx)
+                    count += 1
 
         # cope with packet loss fetching mission
         if mpstate.master().time_since('MISSION_ITEM') >= 2 and mpstate.status.wploader.count() < getattr(mpstate.status.wploader,'expected_count',0):
@@ -1841,12 +1996,7 @@ Auto-detected serial ports are:
 
     # open master link
     for mdev in opts.master:
-        if mdev.startswith('tcp:'):
-            m = mavutil.mavtcp(mdev[4:])
-        elif mdev.find(':') != -1:
-            m = mavutil.mavudp(mdev, input=True)
-        else:
-            m = mavutil.mavserial(mdev, baud=opts.baudrate, autoreconnect=True)
+        m = mavutil.mavlink_connection(mdev, autoreconnect=True, baud=opts.baudrate)
         m.mav.set_callback(master_callback, m)
         if hasattr(m.mav, 'set_send_callback'):
             m.mav.set_send_callback(master_send_callback, m)
